/**
 * \file shape_generation.cpp
 * \brief ****
 * \author Luis Alvarez \n \n
*/


#include <iostream>
#include <vector>
#include <math.h>
#include <time.h>
#include "tree_generation.h"
#include "shape_generation.h"
#include <climits>

#define ami_PI 3.14159265358979323846
#define CERO 1e-12

#define NautomaticShapeDatabase 6
#define NDiskStoredShapeDatabase 3
#define PaletteDataBaseSize 43

static int Ncall_tree_node_generator=0;
static int Ncall_tree_node_generator_parameters=0;

int AutomaticShapeDatabaseSize[NautomaticShapeDatabase]={7,3,1,1,1,1};

#define NumberOfShapes 44
static int DatabaseIndex[NumberOfShapes]={
 0,0,0,0,0,0,0, //7
 1,1,1, //3
 2,3,4,5, //14
 1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000, //18
 1001,1001,1001,1001,1001, //5
 1002,
 1003,
 -2,
 0,
 1,
 1000,
 1001
};
static int ShapeIndex[NumberOfShapes]={0,1,2,3,4,5,6,0,1,2,0,0,0,0, //14
   0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17, //18
   0,1,2,3,4,
   0,
   0,
   -2,
   -1,
   -1,
   -1,
   -1
};

/// check if a string is a number
bool isn(char str[200]){
    int np=0;
    //printf("%s\n",str);
    if(strlen(str)==0) return false;
    if(str[0]=='.') np++;
    else if(isdigit(str[0])==false && str[0]!='-') return false;
    for(int k=1;k<strlen(str);k++){
      if(str[k]=='.') np++;
      else if (isdigit(str[k])==false) return false;
    }
    if(np<=1) return true;
    return false;
}

/// DEFINE SHAPE INDEXES FOR NEW SHAPES UPLOADED BY THE USER
void build_database_and_shape_index(vector<int> &Database_Index,vector<int> &Shape_Index,int NewShapes){
  Database_Index.resize(NumberOfShapes);
  Shape_Index.resize(NumberOfShapes);
  for(int k=0;k<NumberOfShapes;k++){
    Database_Index[k]=DatabaseIndex[k];
    Shape_Index[k]=ShapeIndex[k];
  }
  if(NewShapes<=0) return;
  Database_Index.push_back(1004);
  Shape_Index.push_back(-1);
  for(int k=0;k<NewShapes;k++){
    Database_Index.push_back(1004);
    Shape_Index.push_back(k);
  }
}

/// DEFINE SHAPE DATABASE AND INDEX FROM A SINGLE NUMBER
void tree_node_generator_parameters::shape_database_and_index_definition(int m){
  m=m%NumberOfShapes;
  shape_database=DatabaseIndex[m];
  shape_index=ShapeIndex[m];
}

/// DEFINE SHAPE DATABASE AND INDEX FROM A SINGLE NUMBER
void tree_node_generator_parameters::shape_database_and_index_definition(int m,vector<int> &Database_Index,vector<int> &Shape_Index){
  m=m%Database_Index.size();
  shape_database=Database_Index[m];
  shape_index=Shape_Index[m];
}

 /// DEFINE A SINGLE INDEX NUMBER FROM SHAPE DATABASE AND INDEX
int tree_node_generator_parameters::shape_database_and_index_definition_inverse(int shape_database,int shape_index){
  if(shape_database<0) return 39;
  int Nshapes=0;
  for(int k=0;k<NumberOfShapes-5;k++){
    if(DatabaseIndex[k]==shape_database) Nshapes++;
  }
  if(Nshapes==0){
   printf("database index %d is no recognized\n",shape_database);
   scanf("%d\n",&Nshapes);
  }
  if(shape_index>0) shape_index=shape_index%Nshapes;
  if(Nshapes==1) shape_index=0;
  int index;
  for(index=0;index<NumberOfShapes;index++){
    if(DatabaseIndex[index]==shape_database && (ShapeIndex[index]==shape_index || Nshapes==1) ) break;
  }
  if(index==NumberOfShapes){
    printf("database index %d and shape index = %d are not recognized\n",shape_database,shape_index);
    scanf("%d\n",&Nshapes);
  }
  return index;
}

 /// DEFINE A SINGLE INDEX NUMBER FROM SHAPE DATABASE AND INDEX
int tree_node_generator_parameters::shape_database_and_index_definition_inverse(int shape_database,int shape_index,vector<int> &Database_Index,vector<int> &Shape_Index){
  if(shape_database!=1004) return shape_database_and_index_definition_inverse(shape_database,shape_index);
  int index=NumberOfShapes;
  if(shape_index>=0) index+=shape_index+1;
  return index;
}

tree_node_generator_parameters::tree_node_generator_parameters()
{
  //srand (time(NULL)+Ncall_tree_node_generator_parameters); rand();
  Ncall_tree_node_generator_parameters++;
  random_seed=rand();
  shape_database= ami_rand<0.5?-2:(ami_rand<0.5?rand()%NautomaticShapeDatabase:1000+rand()%NDiskStoredShapeDatabase);
  shape_index=shape_database<0?-1:(ami_rand<0.5?-1:rand());
  max_number_shapes=INT_MAX;
  shape_diameter=0.1;
  shape_distribution_type=ami_rand<0.8?rand()%4:5;
  if(ami_rand<0.4){
    angleInit_1=0.;
    angleFinal_1=0;
    Nangles_1=1;
    angleInit_2=0.;
    angleFinal_2=ami_PI;
    Nangles_2=2;
  }
  else{
    angleInit_1=-ami_rand*ami_PI;
    angleFinal_1=ami_rand*ami_PI;
    Nangles_1=1+360*(angleFinal_1-angleInit_1)/ami_PI;
    angleInit_2=-ami_rand*ami_PI;
    angleFinal_2=ami_rand*ami_PI;
    Nangles_2=1+360*(angleFinal_1-angleInit_1)/ami_PI;
  }
  inclusion_father=ami_rand<0.9?-1:0;
  exclusion_brothers=ami_rand<0.8?-1:0;
  exclusion_others=ami_rand<0.8?-1:0;
  if(ami_rand<0.5){
    min_uniform_scaling=max_uniform_scaling=1.;
  }
  else{
    min_uniform_scaling=0.5+ami_rand;
    max_uniform_scaling=min_uniform_scaling+10.*ami_rand;
    inclusion_father=-1;
    exclusion_brothers=-1;
    exclusion_others=-1;
  }
  if(ami_rand<0.5){
    min_horizontal_scaling=1.;
    max_horizontal_scaling=1.;
  }
  else if(ami_rand<0.5){
    min_horizontal_scaling=0.1+0.9*ami_rand;
    max_horizontal_scaling=1.+4.*ami_rand;
  }
  else{
    min_horizontal_scaling=0.1+0.4*ami_rand;
    max_horizontal_scaling=min_horizontal_scaling;
  }
  if(ami_rand<0.8){
    min_distance_contouring=max_distance_contouring=0.05*ami_rand;
  }
  else{
    min_distance_contouring=0.05*ami_rand;
    max_distance_contouring=min_distance_contouring+0.05*ami_rand;
  }
  if(ami_rand<0.6){
    min_distance_contouring=-min_distance_contouring;
    max_distance_contouring=-max_distance_contouring;
  }
  border_thick=ami_rand<0.7?0.:0.005*ami_rand;
  /// RENDER PARAMETERS
  random_seed_color=rand();
  palette_index=rand()%PaletteDataBaseSize;
  if(ami_rand<0.5){
    min_transparency=max_transparency=0;
  }
  else{
    min_transparency=0.5+0.4*ami_rand;
    max_transparency=min_transparency+(0.9-min_transparency)*ami_rand;
  }
  if(ami_rand<0.5){
    min_light_spot=-1.;
    max_light_spot=-1.;
  }
  else{
    min_light_spot=4.*ami_rand;
    max_light_spot=min_light_spot+4.*ami_rand;
  }

  //point2d light_spot_center; /** Center for the light spot effect */
  tesselation_palette_index=ami_rand<0.5?-1:rand()%PaletteDataBaseSize;
  keepFatherInclusion=ami_rand<0.5?0:-1;

};

void tree_node_generator_parameters::tree_node_generator(shape &S,vector < vector<shape> > &ShapeDataBaseV,vector<imageBMP> &PaletteDataBase){
  //printf("init tree_node_generator()\n");
  //printf("random_seed=%d\n",random_seed); system("pause");
  srand(random_seed+Ncall_tree_node_generator); rand();
  Ncall_tree_node_generator++;

  tree_node_generator_parameters TngpCopy=*this;

  //printf("S nodes : "); for(int n=0;n<S.nodes.size();n++) printf("%d ",S.nodes[n]); printf("\n");
  //printf("  nodes : "); for(int n=0;n<nodes.size();n++) printf("%d ",nodes[n]); printf("\n");

  vector< vector <int> > shapeList;
  if(shape_database==-1){
    double temp=(double) rand()/RAND_MAX;
    //printf("temp=%lf\n",temp);
    if(temp<0.5) shape_database=999*((double) rand()/RAND_MAX);
    else shape_database=1000+((int) (999*((double) rand()/RAND_MAX)))%ShapeDataBaseV.size();
  }
  else if(shape_database==-2){
    vector<int> vt(2);
    for(int k=0;k<NautomaticShapeDatabase;k++){
      vt[0]=k;
      for(int m=0;m<AutomaticShapeDatabaseSize[k];m++){
        vt[1]=m;
        shapeList.push_back(vt);
      }
    }
    for(int k=0;k<ShapeDataBaseV.size();k++){
      vt[0]=1000+k;
      for(int m=0;m<ShapeDataBaseV[k].size();m++){
        vt[1]=m;
        shapeList.push_back(vt);
      }
    }
    rand(); rand();
  }
  else{ rand(); rand();}
  //printf("shape_database=%d\n",shape_database); system("pause");


  if(palette_index==-1){palette_index=PaletteDataBase.size()*((double) rand()/RAND_MAX);}
  else rand();
  palette_index=palette_index%PaletteDataBase.size();
  //printf("palette_index=%d PaletteDataBase.size()=%d\n",palette_index,PaletteDataBase.size()); system("pause");


  //WE CHECK THE NODE
  int n;
  for(n=0;n<S.nodes.size() && n<nodes.size();n++){
    if(S.nodes[n]==nodes[n]) continue;
    break;
  }
  //printf("n=%d\n",n);
  //system("pause");
  if(n!=nodes.size()-1){
    for(int k=0;k<S.shapeChild.size();k++){
      *this=TngpCopy;
      tree_node_generator(S.shapeChild[k],ShapeDataBaseV,PaletteDataBase);
    }
    *this=TngpCopy;
    return;
  }
  vector<point2d> pV;
  vector<int> SelectedTriangles, SelectedSegments;
  vector<point2d> SelectedPoints;
  int Npoints=0;
  int NinitialChilds=S.shapeChild.size();
  int Nshapes=0;
  if(shape_distribution_type==5){
    S.T.boundary_segments(shape_diameter,SelectedTriangles,SelectedSegments,SelectedPoints);
    Npoints=SelectedTriangles.size();
  }
  else if(shape_distribution_type==6){
     shape S2=S.mondrian(shape_diameter,100000);
     for(int k=0;k<S2.shapeChild.size();k++){
       S.shapeChild.push_back(S2.shapeChild[k]);
     }
  }
  else {
    pV=S.T.point_generation(shape_diameter,shape_distribution_type);
    Npoints=pV.size();
  }
  //printf("Npoints=%d\n",Npoints); //system("pause");

  for(int k=0;k<Npoints;k++){
    //printf("%d ",k);
    float zoom=min_uniform_scaling+(max_uniform_scaling-min_uniform_scaling)*rand()/RAND_MAX;
    float angle_1=angleInit_1+((angleFinal_1-angleInit_1)/Nangles_1)*((int)(Nangles_1*((double) rand()/RAND_MAX)));
    float angle_2=angleInit_2+((angleFinal_2-angleInit_2)/Nangles_2)*((int)(Nangles_2*((double) rand()/RAND_MAX)));
    float horizontal_scaling=min_horizontal_scaling+(max_horizontal_scaling-min_horizontal_scaling)*rand()/RAND_MAX;
    affinity A(angle_1,angle_2,zoom*shape_diameter*0.5,horizontal_scaling);
    shape S2;
    if(shape_database==-2){
      int i= ((int) (shapeList.size()*((double) rand()/RAND_MAX)))%shapeList.size();
      if(shapeList[i][0]<1000) S2=shape_generator().new_shape(shapeList[i][0],shapeList[i][1]);
      else S2=ShapeDataBaseV[shapeList[i][0]-1000][shapeList[i][1]];
    }
    else{
      if(shape_database<1000){ /// shape generation on spot
        S2=shape_generator().new_shape(shape_database,shape_index);
        //S2.print();
      }
      else { /// shape generation from a database
        int i=shape_index;
        if(i<0) i=ShapeDataBaseV[shape_database-1000].size()*((double) rand()/RAND_MAX);
        S2=ShapeDataBaseV[(shape_database-1000)%ShapeDataBaseV.size()][i%ShapeDataBaseV[shape_database-1000].size()];
      }
    }
//    /// SYMMETRIZATION OF THE SHAPES
//    {
//      S2.extrema_update();
//      if(S2.extrema.size()==2){
//        point2d  c;
//        float w0=0.5+0.*(double) rand()/RAND_MAX;
//        float w1=0.5+0.*(double) rand()/RAND_MAX;
//        c.x=S2.extrema[0].x*w0+S2.extrema[1].x*(1-w0);
//        c.y=S2.extrema[0].y*w1+S2.extrema[1].y*(1-w1);
//        int Nangles=8;
//        S2.symmetry(c,Nangles);
//      }
//    }
    S2=S2*A;
    //S2.print();
    float distance_contouring;
    if(shape_distribution_type==5){
      distance_contouring=min_distance_contouring+(max_distance_contouring-min_distance_contouring)*rand()/RAND_MAX;
      if(shape_database==1 && (shape_index==1 || shape_index==2) ) {
        if(S2.T.t[0].v[1]!=1 || S2.T.t[0].v[2]!=0 || (S2.T.t[0].v[0]!=2 && S2.T.t[0].v[0]!=3) ) continue;
      }
      affinity A=connectT12T0(S.T,SelectedTriangles[k],SelectedSegments[k],S2.T,0,1,distance_contouring,1,SelectedPoints[k]);
      S2=S2*A;
    }
    else{
      S2=S2+pV[k];
      //S2.print();
      //printf("pV[k].x=%lf,pV[k].y=%lf\n",pV[k].x,pV[k].y);
     // system("pause");
    }

//    if(border_thick>0){
//        shape Sborder(S2.tree_level,S2.T.border(border_thick),color((unsigned char) 0,(unsigned char) 0,(unsigned char) 0));
//        //Sborder.keepFatherInclusion=keepFatherInclusion;
//        Sborder.keepFatherInclusion=-1;
//        S2.shapeBorder.push_back(Sborder);
//    }
    S2.extrema_update();
    //if(nodes.size()==3) {S2.print(); system("pause");}
    if(inclusion_father==0 && (shape_distribution_type!=5 || distance_contouring<0. )) if(S.inclusion_test(S2)==false) continue;
    if(exclusion_brothers==0){
      int k;
      for(k=NinitialChilds;k<S.shapeChild.size();k++){
        if(S2.intersection_test(S.shapeChild[k])==true)  break;
      }
      if(k!=S.shapeChild.size()) continue;
    }
    if(exclusion_others==0){
      int k;
      for(k=0;k<NinitialChilds;k++){
        if(S2.intersection_test(S.shapeChild[k])==true)  break;
      }
      if(k!=NinitialChilds) continue;
    }
    //S2.nodes=nodes;
    //S2.tesselation_palette_index=tesselation_palette_index;

    S.shapeChild.push_back(S2);
    Nshapes++;
    if(Nshapes>=max_number_shapes) break;
  }

  /// RENDER AND OTHER OPTIONS
  srand(random_seed_color+random_seed+Ncall_tree_node_generator); rand();
  //printf("palette_index=%d PaletteDataBase.size()=%d\n",palette_index,PaletteDataBase.size()); //system("pause");
  for(int k=NinitialChilds;k<S.shapeChild.size();k++){
    if(border_thick>0){
        shape Sborder(S.shapeChild[k].tree_level,S.shapeChild[k].T.border(border_thick),color((unsigned char) 0,(unsigned char) 0,(unsigned char) 0));
        //Sborder.keepFatherInclusion=keepFatherInclusion;
        Sborder.keepFatherInclusion=-1;
        S.shapeChild[k].shapeBorder.push_back(Sborder);
    }
    S.shapeChild[k].nodes=nodes;
    S.shapeChild[k].tesselation_palette_index=tesselation_palette_index;
    S.shapeChild[k].set_color(PaletteDataBase[palette_index%PaletteDataBase.size()],S.c,50);
    //printf("S.shapeChild[k].c=(%d,%d,%d)\n",S.shapeChild[k].c.B_,S.shapeChild[k].c.G_,S.shapeChild[k].c.R_);
    float transp=min_transparency +(max_transparency-min_transparency)*rand()/RAND_MAX;
    S.shapeChild[k].set_transparency(transp,-1);
    float li_sp=(min_light_spot +(max_light_spot-min_light_spot)*rand()/RAND_MAX);///(10.*shape_diameter);
    S.shapeChild[k].set_light_spot(li_sp,-1);
    if(S.shapeChild[k].light_spot>=0.) S.shapeChild[k].set_light_spot_center(-1);
    S.shapeChild[k].set_tesselation(tesselation_palette_index,-1);
    S.shapeChild[k].set_keepFatherInclusion(keepFatherInclusion,-1);
  }
  //system("pause");
  //printf("end point processing\n");
  *this=TngpCopy;
}

///// WRITE IN DISK IMAGE PARAMETER CONFIGURATION
//void write_image_configuration_parameters(
//char name[],
//shape &S,
//float &width_float,
//float &height_float,
//vector<float> &border_thick,
//vector<color> &background_color,
//float &zoom,
//vector<tree_node_generator_parameters> &TngpV,
//int &OrderToDrawShapes,
//float &scale,
//string &DefaultPalette)
//{
//  FILE *f;
//  f = fopen (name , "w");
//  if(f==NULL){
//    printf("We can not open file : %s",name);
//    //system("pause");
//    { int ii_; scanf("%d",&ii_);}
//  }
//
//  fprintf(f,"INFORMATION ABOUT FIRST RECTANGLE CONTAINER\n");
////  float width_float=0,height_float=0;
////  for(int k=0;k<S.T.p.size();k++){
////    if(S.T.p[k].x>width_float) width_float=S.T.p[k].x;
////    if(S.T.p[k].y>height_float) height_float=S.T.p[k].y;
////  }
////  printf("border_thick.size()=%d,background_color.size()=%d\n",border_thick.size(),background_color.size());
//  fprintf(f,"NORMALIZED IMAGE DIMENSION\n");
//  fprintf(f,"width_float = %lf\n",width_float);
//  fprintf(f,"height_float = %lf  (the image size 1.413793 x 1. corresponds to A0 horizontal scaling) \n",height_float);
//  fprintf(f,"THICK OF EACH STRIPE OF THE BORDER BAND (PERSIAN CARPETS)\n");
//  fprintf(f,"border_thick = ");
//  if(border_thick.size()>0){
//    for(int n=0;n<border_thick.size()-1;n++) fprintf(f,"%lf:",border_thick[n]);
//    fprintf(f,"%lf  (thick of different stripes are separated by symbol :)\n",border_thick[border_thick.size()-1]);
//  }
//  else{
//     fprintf(f,"%lf  (thick of different stripes are separated by symbol :)\n",0.);
//  }
//  fprintf(f,"RENDER PARAMETERS\n");
//  fprintf(f,"Default color palette = %s (a BMP image with the default color palette. If the file exists it is used to color all shapes)\n",DefaultPalette.c_str());
//  fprintf(f,"BACKGROUND COLOR OF THE RECTANGLE AND THE BORDER STRIPES \n");
//  fprintf(f,"R = ");
//  for(int n=0;n<background_color.size()-1;n++) fprintf(f,"%d:",background_color[n].B_);
//  fprintf(f,"%d (red channel of the rectangle and stripes separated by symbol :)\n",background_color[background_color.size()-1].B_);
//  fprintf(f,"G = ");
//  for(int n=0;n<background_color.size()-1;n++) fprintf(f,"%d:",background_color[n].G_);
//  fprintf(f,"%d (green channel of the rectangle and stripes separated by symbol :)\n",background_color[background_color.size()-1].G_);
//  fprintf(f,"B = ");
//  for(int n=0;n<background_color.size()-1;n++) fprintf(f,"%d:",background_color[n].R_);
//  fprintf(f,"%d (blue channel of the rectangle and stripes separated by symbol :)\n",background_color[background_color.size()-1].R_);
//  fprintf(f,"LIGHT SPOT CONFIGURATION FOR THE RECTANGLE AND THE BORDER STRIPES\n");
//  fprintf(f,"light_spot = %lf  (>0 means light spot effect) \n",S.light_spot);
//  fprintf(f,"light_spot_center.x = %lf\n",S.light_spot_center.x);
//  fprintf(f,"light_spot_center.y = %lf\n",S.light_spot_center.y);
//  fprintf(f,"HEIGHT OF THE GENERATED BMP IMAGE\n");
//  fprintf(f,"bmp image height = %1.0lf\n",zoom);
//  fprintf(f,"ORDER TO DRAW SHAPES IN THE IMAGE\n");
//  fprintf(f,"OrderToDrawShapes = %d (if negative shapes are drawn in a random way. Otherwise they are drawn in the order they have been build) \n",OrderToDrawShapes);
//  fprintf(f,"RESOLUTION SCALE OF THE SHAPES \n");
//  fprintf(f,"ResolutionScale = %lf (>1. lower resolution , <1. higher resolution)\n\n",scale);
//
//  fprintf(f,"SHAPE TREE CONFIGURATION. THE CONTAINER RECTANGLE AND BORDER STRIPES ARE IN THE FIRST TREE LEVEL\n");
//  fprintf(f,"THE RECTANGLE IS THE NODE WITH INDEX 0. THE EXTERNAL BORDER STRIPE IS THE NODE WITH INDEX 1,ETC..\n\n");
//
//  fprintf(f,"NODE CONFIGURATION\n");
//  for(int k=0;k<TngpV.size();k++){
//    //printf("k=%d TngpV.size()=%d\n",k,TngpV.size());
//    if(TngpV[k].nodes.size()<2){
//      printf("Node is not well defined at level : %d\n",k); { int ii_; scanf("%d",&ii_);} //system("pause");
//    }
//    fprintf(f,"depth of the node = ");
//    for(int n=0;n<TngpV[k].nodes.size()-1;n++) fprintf(f,"%d:",TngpV[k].nodes[n]);
//    fprintf(f,"%d  (depth of node index separated by symbol :)\n",TngpV[k].nodes[TngpV[k].nodes.size()-1]);
//    fprintf(f,"random_seed = %d  (random seed to initiate random sequence in this node)\n",TngpV[k].random_seed);
//    fprintf(f,"SHAPE DATABASE INFORMATION USED FOR THIS NODE\n");
//    fprintf(f,"shape_database = %d  (index of the used database, -1 means random choice of a single database, -2 means all databases are combined)\n",TngpV[k].shape_database);
//    fprintf(f,"shape_index = %d  (index of the shape in the database, -1 means random choice)\n",TngpV[k].shape_index);
//    fprintf(f,"SHAPE LOCATION DISTRIBUTION IN THIS NODE\n");
//    fprintf(f,"shape_diameter = %lf (default shape diameter. The number of shapes to include is computed to cover the father shape using such shape diameter. The smaller the shape diameter, the larger the number of shapes)\n",TngpV[k].shape_diameter);
//    fprintf(f,"shape_distribution_type = %d  (0: random, 1: square, 2: hexagon, 3: Delauny, 4: Stripe Border Centerline 5: Contouring)\n",TngpV[k].shape_distribution_type);
//    fprintf(f,"contouring parameter (only applies for shape_distribution_type equals to 5) distance_contouring>0 : contouring outside the object (<0 inside)\n");
//    fprintf(f,"min_distance_contouring = %lf  (max distance between the contouring shape and the shape to be contoured)\n",TngpV[k].min_distance_contouring);
//    fprintf(f,"max_distance_contouring = %lf  (max distance between the contouring shape and the shape to be contoured)\n",TngpV[k].max_distance_contouring);
//    fprintf(f,"AFFINE TRANSFORMATION PARAMETERS TO APPLY TO EACH SHAPE IN THIS NODE\n");
//    fprintf(f,"angleInit_1 = %lf  (min shape rotation angle)\n",TngpV[k].angleInit_1);
//    fprintf(f,"angleFinal_1 = %lf  (max shape rotation angle)\n",TngpV[k].angleFinal_1);
//    fprintf(f,"Nangles_1 = %d  (number of angles to sort between angleInit and angleFinal)\n",TngpV[k].Nangles_1);
//    fprintf(f,"min_horizontal_scaling = %lf  (min horizontal scaling to apply to the shape)\n",TngpV[k].min_horizontal_scaling);
//    fprintf(f,"max_horizontal_scaling = %lf  (max horizontal scaling to apply to the shape)\n",TngpV[k].max_horizontal_scaling);
//    fprintf(f,"angleInit_2 = %lf  (min shape rotation angle)\n",TngpV[k].angleInit_2);
//    fprintf(f,"angleFinal_2 = %lf  (max shape rotation angle)\n",TngpV[k].angleFinal_2);
//    fprintf(f,"Nangles_2 = %d  (number of angles to sort between angleInit and angleFinal)\n",TngpV[k].Nangles_2);
//    fprintf(f,"min_uniform_scaling = %lf  (min uniform scaling to apply to the default shape size)\n",TngpV[k].min_uniform_scaling);
//    fprintf(f,"max_uniform_scaling = %lf  (max uniform scaling to apply to the default shape size)\n",TngpV[k].max_uniform_scaling);
//    fprintf(f,"SHAPE INTERACTION PARAMETERS\n");
//    fprintf(f,"inclusion_father = %d  (0: activated, -1: no activated) \n",TngpV[k].inclusion_father);
//    fprintf(f,"exclusion_brothers = %d (0: activated, -1: no activated) \n",TngpV[k].exclusion_brothers);
//    fprintf(f,"exclusion_others = %d  (0: activated, -1: no activated)\n",TngpV[k].exclusion_others);
//    fprintf(f,"RENDER PARAMETERS\n");
//    fprintf(f,"random_seed_color = %d  (random seed to initiate random sequence when color assignement)\n",TngpV[k].random_seed_color);
//    fprintf(f,"border_thick = %lf (: >0 a black border is delineated in the shape boundary) \n",TngpV[k].border_thick);
//    fprintf(f,"palette_index = %d  (index of palette in the Palette Database (-1 means random choice)\n",TngpV[k].palette_index);
//    fprintf(f,"min_transparency = %lf\n",TngpV[k].min_transparency);
//    fprintf(f,"max_transparency = %lf\n",TngpV[k].max_transparency);
//    fprintf(f,"min_light_spot = %lf\n",TngpV[k].min_light_spot);
//    fprintf(f,"max_light_spot = %lf\n",TngpV[k].max_light_spot);
//    fprintf(f,"tesselation_palette_index = %d  (if <0 no tesselation, otherwise is the palette index to color the shape)\n",TngpV[k].tesselation_palette_index);
//    fprintf(f,"keepFatherInclusion = %d\n\n",TngpV[k].keepFatherInclusion);
//
//  }
//  fclose(f);
//}

/// WRITE IN DISK IMAGE PARAMETER CONFIGURATION
void write_image_configuration_parameters(
char name[],
shape &S,
float &width_float,
float &height_float,
vector<float> &border_thick,
vector<color> &background_color,
float &zoom,
vector<tree_node_generator_parameters> &TngpV,
int &OrderToDrawShapes,
float &scale,
string &DefaultPalette)
{
  FILE *f;
  f = fopen (name , "w");
  if(f==NULL){
    printf("We can not open file : %s",name);
    //system("pause");
    { int ii_; scanf("%d",&ii_);}
  }

  //fprintf(f,"INFORMATION ABOUT FIRST RECTANGLE CONTAINER\n");
//  float width_float=0,height_float=0;
//  for(int k=0;k<S.T.p.size();k++){
//    if(S.T.p[k].x>width_float) width_float=S.T.p[k].x;
//    if(S.T.p[k].y>height_float) height_float=S.T.p[k].y;
//  }
//  printf("border_thick.size()=%d,background_color.size()=%d\n",border_thick.size(),background_color.size());
//  fprintf(f,"NORMALIZED IMAGE DIMENSION\n");
  fprintf(f,"IMAGE ASPECT RATIO = %lf  (the ratio between the image width and height)\n",width_float);
  fprintf(f,"NORMALIZED IMAGE HEIGHT = %lf (The normalized image height is always 1.  All other length  parameters in the image (point coordinates, diameters, thickness, distance) refer to this unit.)\n",height_float);
  //fprintf(f,"THICK OF EACH STRIPE OF THE BORDER BAND (PERSIAN CARPETS)\n");
  fprintf(f,"BORDER THICKNESS  = ");
  if(border_thick.size()>0){
    for(int n=0;n<border_thick.size()-1;n++) fprintf(f,"%1.3lf:",border_thick[n]);
    fprintf(f,"%1.3lf (thickness of Persian carpet type stripes. They are separated by symbol :, for example 0.02:0.04:0.02)\n",border_thick[border_thick.size()-1]);
  }
  else{
     fprintf(f,"%1.3lf  (thick of different stripes are separated by symbol :)\n",0.);
  }
  //fprintf(f,"RENDER PARAMETERS\n");
  fprintf(f,"DEFAULT COLOR PALETTE IMAGE = %s (an image with the default color palette. It is optional, if the file exists it is used to color all shapes)\n",DefaultPalette.c_str());
  fprintf(f,"BACKGROUND COLOR OF THE CONTAINER RECTANGLE AND THE BORDER STRIPES \n");
  fprintf(f,"R = ");
  for(int n=0;n<background_color.size()-1;n++) fprintf(f,"%d:",background_color[n].B_);
  fprintf(f,"%d (red channel of the rectangle and stripes separated by symbol :)\n",background_color[background_color.size()-1].B_);
  fprintf(f,"G = ");
  for(int n=0;n<background_color.size()-1;n++) fprintf(f,"%d:",background_color[n].G_);
  fprintf(f,"%d (green channel of the rectangle and stripes separated by symbol :)\n",background_color[background_color.size()-1].G_);
  fprintf(f,"B = ");
  for(int n=0;n<background_color.size()-1;n++) fprintf(f,"%d:",background_color[n].R_);
  fprintf(f,"%d (blue channel of the rectangle and stripes separated by symbol :)\n",background_color[background_color.size()-1].R_);
  //fprintf(f,"LIGHT SPOT CONFIGURATION FOR THE RECTANGLE AND THE BORDER STRIPES\n");
  fprintf(f,"LIGHT SPOT OF THE CONTAINER RECTANGLE  = %1.3lf  (if nonpositive no light spot effect is applied, otherwise usual value ranges from 0.5 to 8.) \n",S.light_spot);
  fprintf(f,"LIGHT SPOT CENTER x COORDINATE = %1.2lf\n",S.light_spot_center.x);
  fprintf(f,"LIGHT SPOT CENTER y COORDINATE = %1.2lf\n",S.light_spot_center.y);
  fprintf(f,"HEIGHT OF THE GENERATED IMAGE IN PIXELS = %1.0lf\n",zoom);
  fprintf(f,"ORDER TO DRAW SHAPES = %d (if negative shapes are drawn in a random way. Otherwise they are drawn in the order they have been built) \n",OrderToDrawShapes);
  //fprintf(f,"RESOLUTION SCALE OF THE SHAPES \n");
  fprintf(f,"RESOLUTION OF THE TEXTURE = %1.2lf (the lower the resolution the smaller the shapes, the default value is 1, it can range from 0.1 to 4.)\n\n",scale);

//  fprintf(f,"SHAPE TREE CONFIGURATION. THE CONTAINER RECTANGLE AND BORDER STRIPES ARE IN THE FIRST TREE LEVEL\n");
//  fprintf(f,"THE RECTANGLE IS THE NODE WITH INDEX 0. THE EXTERNAL BORDER STRIPE IS THE NODE WITH INDEX 1,ETC..\n\n");

  fprintf(f,"TREE NODE CONFIGURATION\n");
  for(int k=0;k<TngpV.size();k++){
    //printf("k=%d TngpV.size()=%d\n",k,TngpV.size());
    if(TngpV[k].nodes.size()<2){
      printf("Node is not well defined at level : %d\n",k); { int ii_; scanf("%d",&ii_);} //system("pause");
    }
    fprintf(f,"DEPTH OF THE NODE = ");
    for(int n=0;n<TngpV[k].nodes.size()-1;n++) fprintf(f,"%d:",TngpV[k].nodes[n]);
    fprintf(f,"%d  (depth of node index separated by symbol :)\n",TngpV[k].nodes[TngpV[k].nodes.size()-1]);

//    fprintf(f,"RANDOM SEED = %d  (random seed to initiate random sequence in this node)\n",TngpV[k].random_seed);
//    fprintf(f,"SHAPE DATABASE = %d  (index of the used database, -1 means random choice of a single database, -2 means all databases are combined)\n",TngpV[k].shape_database);
//    fprintf(f,"SHAPE INDEX = %d  (index of the shape in the database, -1 means random choice)\n",TngpV[k].shape_index);

    fprintf(f,"RANDOM SEED = %d  (random seed to initiate random sequence in this node ( = )) \n",TngpV[k].random_seed);
    fprintf(f,"BASIC SHAPE = %d  (index in the basic shape collection in the OnLine interface )\n",TngpV[k].shape_database_and_index_definition_inverse(TngpV[k].shape_database,TngpV[k].shape_index));

    //fprintf(f,"SHAPE LOCATION DISTRIBUTION IN THIS NODE\n");
    fprintf(f,"SHAPE DIAMETER = %1.3lf (default shape diameter size. The number of shapes to include is computed to cover the father shape using such shape diameter. The smaller the shape diameter, the larger the number of shapes)\n",TngpV[k].shape_diameter);
    fprintf(f,"SHAPE LOCATION DISTRIBUTION = %d  (0: random, 1: square grid, 2: hexagon grid, 3: Delaunay, 4: shapes centered in stripes of Persian carpets 5: contouring 6: Mondrian type)\n",TngpV[k].shape_distribution_type);
    //fprintf(f,"contouring parameter (only applies for shape_distribution_type equals to 5) distance_contouring>0 : contouring outside the object (<0 inside)\n");
    fprintf(f,"MIN CONTOURING DISTANCE = %lf  (min distance between the contouring shape and the shape to be contoured, only applies for shape location distribution equals to 5),  distance_contouring>0 : contouring outside the object (<0 inside)\n",TngpV[k].min_distance_contouring);
    fprintf(f,"MAX CONTOURING DISTANCE = %lf  (max distance between the contouring shape and the shape to be contoured, only applies for shape location distribution equals to 5),  distance_contouring>0 : contouring outside the object (<0 inside)\n",TngpV[k].max_distance_contouring);
    //fprintf(f,"AFFINE TRANSFORMATION PARAMETERS TO APPLY TO EACH SHAPE IN THIS NODE\n");
    fprintf(f,"ROTATION 1. MIN ANGLE = %lf  (affine transformation. min shape rotation angle (in radians))\n",TngpV[k].angleInit_1);
    fprintf(f,"ROTATION 1. MAX ANGLE = %lf  (affine transformation. max shape rotation angle (in radians))\n",TngpV[k].angleFinal_1);
    fprintf(f,"ROTATION 1. NUMBER OF ANGLES = %d  (affine transformation. number of angles to sort for rotation 1)\n",TngpV[k].Nangles_1);
    fprintf(f,"MIN HORIZONTAL SCALING = %1.3lf  (affine transformation. min horizontal scaling to apply to the shape)\n",TngpV[k].min_horizontal_scaling);
    fprintf(f,"MAX HORIZONTAL SCALING = %1.3lf  (affine transformation. max horizontal scaling to apply to the shape)\n",TngpV[k].max_horizontal_scaling);
    fprintf(f,"ROTATION 2. MIN ANGLE = %lf  (affine transformation. min shape rotation angle (in radians))\n",TngpV[k].angleInit_2);
    fprintf(f,"ROTATION 2. MAX ANGLE = %lf  (affine transformation. max shape rotation angle (in radians))\n",TngpV[k].angleFinal_2);
    fprintf(f,"ROTATION 2. NUMBER OF ANGLES = %d  (affine transformation. number of angles to sort for rotation 2)\n",TngpV[k].Nangles_2);
    fprintf(f,"MIN UNIFORM SCALING = %1.3lf  (affine transformation. min uniform scaling to apply to the default shape size)\n",TngpV[k].min_uniform_scaling);
    fprintf(f,"MAX UNIFORM SCALING = %1.3lf  (affine transformation. max uniform scaling to apply to the default shape size)\n",TngpV[k].max_uniform_scaling);
    fprintf(f,"INCLUSION FATHER = %d  (shape interaction. 0: activated, -1: no activated) \n",TngpV[k].inclusion_father);
    fprintf(f,"EXCLUSION BROTHERS IN THE SAME NODE = %d (shape interaction. 0: activated, -1: no activated) \n",TngpV[k].exclusion_brothers);
    fprintf(f,"EXCLUSION BROTHERS IN OTHER NODES = %d  (shape interaction. 0: activated, -1: no activated)\n",TngpV[k].exclusion_others);
    fprintf(f,"RANDOM SEED COLOR = %d  (random seed to initiate random sequence when color assignment)\n",TngpV[k].random_seed_color);
    fprintf(f,"BORDER THICK = %lf (: >0 a black border is delineated in the shape boundary) \n",TngpV[k].border_thick);
    fprintf(f,"PALETTE INDEX = %d  (index of palette in the default palette database )\n",TngpV[k].palette_index);
    fprintf(f,"MIN TRANSPARENCY = %1.3lf (value ranges from 0 (opaque) to 1 (full transparency))\n",TngpV[k].min_transparency);
    fprintf(f,"MAX TRANSPARENCY = %1.3lf (value ranges from 0 (opaque) to 1 (full transparency))\n",TngpV[k].max_transparency);
    fprintf(f,"MIN LIGHT SPOT = %1.3lf (if nonpositive no light spot effect is applied, otherwise usual value ranges from 0.5 to 8.)\n",TngpV[k].min_light_spot);
    fprintf(f,"MAX LIGHT SPOT = %1.3lf (if nonpositive no light spot effect is applied, otherwise usual value ranges from 0.5 to 8.)\n",TngpV[k].max_light_spot);
    fprintf(f,"TESSELLATION PALETTE INDEX = %d  (if <0 no tessellation, otherwise it is the palette index to color the shape with tessellation)\n",TngpV[k].tesselation_palette_index);
    fprintf(f,"KEEP FATHER INCLUSION = %d (0: shapes are not drawn outside its father, -1: the opposite)\n\n",TngpV[k].keepFatherInclusion);

  }
  fclose(f);
}

/// WRITE IN DISK IMAGE PARAMETER CONFIGURATION
void write_image_configuration_parameters(
char name[],
shape &S,
float &width_float,
float &height_float,
vector<float> &border_thick,
vector<color> &background_color,
float &zoom,
vector<tree_node_generator_parameters> &TngpV,
int &OrderToDrawShapes,
float &scale,
string &DefaultPalette,
vector<int> &Database_Index,
vector<int> &Shape_Index)
{
  FILE *f;
  f = fopen (name , "w");
  if(f==NULL){
    printf("We can not open file : %s",name);
    //system("pause");
    { int ii_; scanf("%d",&ii_);}
  }

  //fprintf(f,"INFORMATION ABOUT FIRST RECTANGLE CONTAINER\n");
//  float width_float=0,height_float=0;
//  for(int k=0;k<S.T.p.size();k++){
//    if(S.T.p[k].x>width_float) width_float=S.T.p[k].x;
//    if(S.T.p[k].y>height_float) height_float=S.T.p[k].y;
//  }
//  printf("border_thick.size()=%d,background_color.size()=%d\n",border_thick.size(),background_color.size());
//  fprintf(f,"NORMALIZED IMAGE DIMENSION\n");
  fprintf(f,"IMAGE ASPECT RATIO = %lf  (the ratio between the image width and height)\n",width_float);
  fprintf(f,"NORMALIZED IMAGE HEIGHT = %lf (The normalized image height is always 1.  All other length  parameters in the image (point coordinates, diameters, thickness, distance) refer to this unit.)\n",height_float);
  //fprintf(f,"THICK OF EACH STRIPE OF THE BORDER BAND (PERSIAN CARPETS)\n");
  fprintf(f,"BORDER THICKNESS  = ");
  if(border_thick.size()>0){
    for(int n=0;n<border_thick.size()-1;n++) fprintf(f,"%1.3lf:",border_thick[n]);
    fprintf(f,"%1.3lf (thickness of Persian carpet type stripes. They are separated by symbol :, for example 0.02:0.04:0.02)\n",border_thick[border_thick.size()-1]);
  }
  else{
     fprintf(f,"%1.3lf  (thick of different stripes are separated by symbol :)\n",0.);
  }
  //fprintf(f,"RENDER PARAMETERS\n");
  fprintf(f,"DEFAULT COLOR PALETTE IMAGE = %s (an image with the default color palette. It is optional, if the file exists it is used to color all shapes)\n",DefaultPalette.c_str());
  fprintf(f,"BACKGROUND COLOR OF THE CONTAINER RECTANGLE AND THE BORDER STRIPES \n");
  fprintf(f,"R = ");
  for(int n=0;n<background_color.size()-1;n++) fprintf(f,"%d:",background_color[n].B_);
  fprintf(f,"%d (red channel of the rectangle and stripes separated by symbol :)\n",background_color[background_color.size()-1].B_);
  fprintf(f,"G = ");
  for(int n=0;n<background_color.size()-1;n++) fprintf(f,"%d:",background_color[n].G_);
  fprintf(f,"%d (green channel of the rectangle and stripes separated by symbol :)\n",background_color[background_color.size()-1].G_);
  fprintf(f,"B = ");
  for(int n=0;n<background_color.size()-1;n++) fprintf(f,"%d:",background_color[n].R_);
  fprintf(f,"%d (blue channel of the rectangle and stripes separated by symbol :)\n",background_color[background_color.size()-1].R_);
  //fprintf(f,"LIGHT SPOT CONFIGURATION FOR THE RECTANGLE AND THE BORDER STRIPES\n");
  fprintf(f,"LIGHT SPOT OF THE CONTAINER RECTANGLE  = %1.3lf  (if nonpositive no light spot effect is applied, otherwise usual value ranges from 0.5 to 8.) \n",S.light_spot);
  fprintf(f,"LIGHT SPOT CENTER x COORDINATE = %1.2lf\n",S.light_spot_center.x);
  fprintf(f,"LIGHT SPOT CENTER y COORDINATE = %1.2lf\n",S.light_spot_center.y);
  fprintf(f,"HEIGHT OF THE GENERATED IMAGE IN PIXELS = %1.0lf\n",zoom);
  fprintf(f,"ORDER TO DRAW SHAPES = %d (if negative shapes are drawn in a random way. Otherwise they are drawn in the order they have been built) \n",OrderToDrawShapes);
  //fprintf(f,"RESOLUTION SCALE OF THE SHAPES \n");
  fprintf(f,"RESOLUTION OF THE TEXTURE = %1.2lf (the lower the resolution the smaller the shapes, the default value is 1, it can range from 0.1 to 4.)\n\n",scale);

//  fprintf(f,"SHAPE TREE CONFIGURATION. THE CONTAINER RECTANGLE AND BORDER STRIPES ARE IN THE FIRST TREE LEVEL\n");
//  fprintf(f,"THE RECTANGLE IS THE NODE WITH INDEX 0. THE EXTERNAL BORDER STRIPE IS THE NODE WITH INDEX 1,ETC..\n\n");

  fprintf(f,"TREE NODE CONFIGURATION\n");
  for(int k=0;k<TngpV.size();k++){
    //printf("k=%d TngpV.size()=%d\n",k,TngpV.size());
    if(TngpV[k].nodes.size()<2){
      printf("Node is not well defined at level : %d\n",k); { int ii_; scanf("%d",&ii_);} //system("pause");
    }
    fprintf(f,"DEPTH OF THE NODE = ");
    for(int n=0;n<TngpV[k].nodes.size()-1;n++) fprintf(f,"%d:",TngpV[k].nodes[n]);
    fprintf(f,"%d  (depth of node index separated by symbol :)\n",TngpV[k].nodes[TngpV[k].nodes.size()-1]);

//    fprintf(f,"RANDOM SEED = %d  (random seed to initiate random sequence in this node)\n",TngpV[k].random_seed);
//    fprintf(f,"SHAPE DATABASE = %d  (index of the used database, -1 means random choice of a single database, -2 means all databases are combined)\n",TngpV[k].shape_database);
//    fprintf(f,"SHAPE INDEX = %d  (index of the shape in the database, -1 means random choice)\n",TngpV[k].shape_index);

    fprintf(f,"RANDOM SEED = %d  (random seed to initiate random sequence in this node ( = )) \n",TngpV[k].random_seed);
    fprintf(f,"BASIC SHAPE = %d  (index in the basic shape collection in the OnLine interface )\n",TngpV[k].shape_database_and_index_definition_inverse(TngpV[k].shape_database,TngpV[k].shape_index,Database_Index,Shape_Index));

    //fprintf(f,"SHAPE LOCATION DISTRIBUTION IN THIS NODE\n");
    fprintf(f,"SHAPE DIAMETER = %1.3lf (default shape diameter size. The number of shapes to include is computed to cover the father shape using such shape diameter. The smaller the shape diameter, the larger the number of shapes)\n",TngpV[k].shape_diameter);
    fprintf(f,"SHAPE LOCATION DISTRIBUTION = %d  (0: random, 1: square grid, 2: hexagon grid, 3: Delaunay, 4: shapes centered in stripes of Persian carpets 5: contouring 6: Mondrian type)\n",TngpV[k].shape_distribution_type);
    //fprintf(f,"contouring parameter (only applies for shape_distribution_type equals to 5) distance_contouring>0 : contouring outside the object (<0 inside)\n");
    fprintf(f,"MIN CONTOURING DISTANCE = %lf  (min distance between the contouring shape and the shape to be contoured, only applies for shape location distribution equals to 5),  distance_contouring>0 : contouring outside the object (<0 inside)\n",TngpV[k].min_distance_contouring);
    fprintf(f,"MAX CONTOURING DISTANCE = %lf  (max distance between the contouring shape and the shape to be contoured, only applies for shape location distribution equals to 5),  distance_contouring>0 : contouring outside the object (<0 inside)\n",TngpV[k].max_distance_contouring);
    //fprintf(f,"AFFINE TRANSFORMATION PARAMETERS TO APPLY TO EACH SHAPE IN THIS NODE\n");
    fprintf(f,"ROTATION 1. MIN ANGLE = %lf  (affine transformation. min shape rotation angle (in radians))\n",TngpV[k].angleInit_1);
    fprintf(f,"ROTATION 1. MAX ANGLE = %lf  (affine transformation. max shape rotation angle (in radians))\n",TngpV[k].angleFinal_1);
    fprintf(f,"ROTATION 1. NUMBER OF ANGLES = %d  (affine transformation. number of angles to sort for rotation 1)\n",TngpV[k].Nangles_1);
    fprintf(f,"MIN HORIZONTAL SCALING = %1.3lf  (affine transformation. min horizontal scaling to apply to the shape)\n",TngpV[k].min_horizontal_scaling);
    fprintf(f,"MAX HORIZONTAL SCALING = %1.3lf  (affine transformation. max horizontal scaling to apply to the shape)\n",TngpV[k].max_horizontal_scaling);
    fprintf(f,"ROTATION 2. MIN ANGLE = %lf  (affine transformation. min shape rotation angle (in radians))\n",TngpV[k].angleInit_2);
    fprintf(f,"ROTATION 2. MAX ANGLE = %lf  (affine transformation. max shape rotation angle (in radians))\n",TngpV[k].angleFinal_2);
    fprintf(f,"ROTATION 2. NUMBER OF ANGLES = %d  (affine transformation. number of angles to sort for rotation 2)\n",TngpV[k].Nangles_2);
    fprintf(f,"MIN UNIFORM SCALING = %1.3lf  (affine transformation. min uniform scaling to apply to the default shape size)\n",TngpV[k].min_uniform_scaling);
    fprintf(f,"MAX UNIFORM SCALING = %1.3lf  (affine transformation. max uniform scaling to apply to the default shape size)\n",TngpV[k].max_uniform_scaling);
    fprintf(f,"INCLUSION FATHER = %d  (shape interaction. 0: activated, -1: no activated) \n",TngpV[k].inclusion_father);
    fprintf(f,"EXCLUSION BROTHERS IN THE SAME NODE = %d (shape interaction. 0: activated, -1: no activated) \n",TngpV[k].exclusion_brothers);
    fprintf(f,"EXCLUSION BROTHERS IN OTHER NODES = %d  (shape interaction. 0: activated, -1: no activated)\n",TngpV[k].exclusion_others);
    fprintf(f,"RANDOM SEED COLOR = %d  (random seed to initiate random sequence when color assignment)\n",TngpV[k].random_seed_color);
    fprintf(f,"BORDER THICK = %lf (: >0 a black border is delineated in the shape boundary) \n",TngpV[k].border_thick);
    fprintf(f,"PALETTE INDEX = %d  (index of palette in the default palette database )\n",TngpV[k].palette_index);
    fprintf(f,"MIN TRANSPARENCY = %1.3lf (value ranges from 0 (opaque) to 1 (full transparency))\n",TngpV[k].min_transparency);
    fprintf(f,"MAX TRANSPARENCY = %1.3lf (value ranges from 0 (opaque) to 1 (full transparency))\n",TngpV[k].max_transparency);
    fprintf(f,"MIN LIGHT SPOT = %1.3lf (if nonpositive no light spot effect is applied, otherwise usual value ranges from 0.5 to 8.)\n",TngpV[k].min_light_spot);
    fprintf(f,"MAX LIGHT SPOT = %1.3lf (if nonpositive no light spot effect is applied, otherwise usual value ranges from 0.5 to 8.)\n",TngpV[k].max_light_spot);
    fprintf(f,"TESSELLATION PALETTE INDEX = %d  (if <0 no tessellation, otherwise it is the palette index to color the shape with tessellation)\n",TngpV[k].tesselation_palette_index);
    fprintf(f,"KEEP FATHER INCLUSION = %d (0: shapes are not drawn outside its father, -1: the opposite)\n\n",TngpV[k].keepFatherInclusion);

  }
  fclose(f);
}

/// READ FROM DISK IMAGE PARAMETER CONFIGURATION
int read_image_configuration_parameters(
char name[],
shape &S,
float &width_float,
float &height_float,
vector<float> &border_thick,
vector<color> &background_color,
float &zoom,
vector<tree_node_generator_parameters> &TngpV,
int &OrderToDrawShapes,
float &scale,
string &DefaultPalette)
{
  FILE *f;
  f = fopen (name , "r");
  if(f==NULL){
    printf("We can not open file : %s",name);
    //system("pause");
    { int ii_; scanf("%d",&ii_);}
  }

  TngpV.clear();

  char nod[200],strp[200],c='o';

  while(c!='=' && c!=EOF) {c=getc(f);  } c='o';
  fscanf(f,"%s",strp); //printf("strp = %s\n",strp); system("pause");
  width_float=atof(strp);

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  height_float=atof(strp);

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
    //printf("%s\n",strp);
    //printf("length stpr=%d\n",strlen(strp));
  border_thick.clear();
  for(int k=0;k<strlen(strp);k++){
     int m=0;
     while(strp[k]!=':' && k<strlen(strp)){
      nod[m++]=strp[k++];
     }
     nod[m]='\0';
     //printf("%s\n",nod);
     border_thick.push_back(atof(nod));
  }

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  DefaultPalette= string(strp);

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  //printf("%s\n",strp);
  //printf("length stpr=%d\n",strlen(strp));
  background_color.clear();
  for(int k=0;k<strlen(strp);k++){
     int m=0;
     while(strp[k]!=':' && k<strlen(strp)){
      nod[m++]=strp[k++];
     }
     nod[m]='\0';
     //printf("%s\n",nod);
     background_color.resize(background_color.size()+1);
     background_color[background_color.size()-1].B_=(unsigned char) atoi(nod);
  }
  int ncolors=0;
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  for(int k=0;k<strlen(strp);k++){
     int m=0;
     while(strp[k]!=':' && k<strlen(strp)){
      nod[m++]=strp[k++];
     }
     nod[m]='\0';
     if(ncolors<background_color.size()){
       background_color[ncolors++].G_=(unsigned char) atoi(nod);
     }
  }
  ncolors=0;
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  for(int k=0;k<strlen(strp);k++){
     int m=0;
     while(strp[k]!=':' && k<strlen(strp)){
      nod[m++]=strp[k++];
     }
     nod[m]='\0';
     if(ncolors<background_color.size()){
       background_color[ncolors++].R_=(unsigned char) atoi(nod);
     }
  }
//  for(int k=0;k<background_color.size();k++){
//    printf("c[%d]=(%d,%d,%d)\n",k,background_color[k].R_,background_color[k].G_,background_color[k].B_);
//  }
//  system("pause");

  /// The initial shape is a rectangle with borders
  //S=shape(triangulation(point2d(0.,0.),point2d(width_float,height_float)));
  //S.nodes.push_back(0);
  S.T=triangulation(); /** triangles defining the shape */
  S.extrema.clear(); /** all shape tree bounding box :  extrema[0]=(xmin,ymin) extrema[1]=(xmax,ymax) */
  S.extrema.push_back(point2d(0.,0.));
  S.extrema.push_back(point2d(width_float,height_float));
  S.shapeChild.clear(); /** children shapes */
  S.shapeBorder.clear(); /** Shapes which define the border */
  S.nodes.clear();
  float Sum_border_thick=0;
  for(int k=0;k<border_thick.size();k++) Sum_border_thick+=border_thick[k];
  point2d pmin=point2d(Sum_border_thick,Sum_border_thick);
  point2d pmax=point2d(width_float,height_float)+point2d(-Sum_border_thick,-Sum_border_thick);
  S.shapeChild.push_back(shape(triangulation(pmin,pmax)));
  S.shapeChild[0].nodes.push_back(0);
  S.shapeChild[0].keepFatherInclusion = 0;
  pmin=point2d(0.,0.);
  pmax=point2d(width_float,height_float);
  for(int k=0;k<border_thick.size();k++){
    if(border_thick[k]<=0.) continue;
    S.shapeChild.push_back(shape(triangulation(border_thick[k],pmin,pmax)));
    S.shapeChild[S.shapeChild.size()-1].nodes.push_back(k+1);
    pmin=pmin+point2d(border_thick[k],border_thick[k]);
    pmax=pmax-point2d(border_thick[k],border_thick[k]);
    //S.shapeChild[S.shapeChild.size()-1].c.random();
    S.shapeChild[S.shapeChild.size()-1].keepFatherInclusion = 0;
  }
  //S.print(); system("pause");


//  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
//  fscanf(f,"%s",strp);
//  unsigned char cB=atoi(strp);
//
//  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
//  fscanf(f,"%s",strp);
//  unsigned char cG=atoi(strp);
//
//  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
//  fscanf(f,"%s",strp);
//  unsigned char cR=atoi(strp);

  for(int k=0;k<background_color.size() && k<S.shapeChild.size();k++){ S.shapeChild[k].c=background_color[k]; }

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  S.light_spot=atof(strp);
  for(int k=0;k<S.shapeChild.size();k++) S.shapeChild[k].light_spot=atof(strp);

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  float px=atof(strp);

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  float py=atof(strp);
  S.light_spot_center=point2d(px,py);
  for(int k=0;k<S.shapeChild.size();k++) S.shapeChild[k].light_spot_center=point2d(px,py);

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  zoom=atof(strp);

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  OrderToDrawShapes=atoi(strp);

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  scale=atof(strp);
  printf("scale=%lf\n",scale);


  /// tree_node_generator_parameters
  while(true){
    while(c!='=' && c!=EOF) {c=getc(f);}
    if(c==EOF){
      fclose(f);
      return -1;
    }
    c='o';
    TngpV.resize(TngpV.size()+1);
    int N=TngpV.size()-1;

    fscanf(f,"%s",strp);
    //printf("%s\n",strp);
    //printf("length stpr=%d\n",strlen(strp));
    TngpV[N].nodes.clear();
    for(int k=0;k<strlen(strp);k++){
       int m=0;
       while(strp[k]!=':' && k<strlen(strp)){
        nod[m++]=strp[k++];
       }
       nod[m]='\0';
       //printf("%s\n",nod);
       TngpV[N].nodes.push_back(atoi(nod));
    }
    for(int i=0;i<TngpV[N].nodes.size();i++) printf("%d ",TngpV[N].nodes[i]); //system("pause");

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].random_seed=atoi(strp);
    printf("TngpV[N].random_seed=%d\n",TngpV[N].random_seed);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if( isdigit(strp[0])==true || strp[0]=='-'){
      TngpV[N].shape_database=atoi(strp);
      printf("TngpV[N].shape_database=%d\n",TngpV[N].shape_database);

      while(c!='=' && c!=EOF) {c=getc(f);} c='o';
      fscanf(f,"%s",strp);
      TngpV[N].shape_index=atoi(strp);
      printf("TngpV[N].shape_index=%d\n",TngpV[N].shape_index);
    }
    else{
      while(c!='=' && c!=EOF) {c=getc(f);} c='o';
      fscanf(f,"%s",strp);
      TngpV[N].shape_database_and_index_definition(atoi(strp)%NumberOfShapes);
      printf("TngpV[N].shape_database=%d\n",TngpV[N].shape_database);
      printf("TngpV[N].shape_index=%d\n",TngpV[N].shape_index);
    }

//    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
//    fscanf(f,"%s",strp);
//    TngpV[N].max_number_shapes=atoi(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].shape_diameter=atof(strp);
    printf("TngpV[N].shape_diameter=%lf\n",TngpV[N].shape_diameter);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].shape_distribution_type=atoi(strp);
    printf("TngpV[N].shape_distribution_type=%d\n",TngpV[N].shape_distribution_type);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].min_distance_contouring=atof(strp);
    printf("TngpV[N].min_distance_contouring=%lf\n",TngpV[N].min_distance_contouring);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].max_distance_contouring=atof(strp);
    printf("TngpV[N].max_distance_contouring=%lf\n",TngpV[N].max_distance_contouring);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].angleInit_1=atof(strp);
    printf("TngpV[N].angleInit_1=%lf\n",TngpV[N].angleInit_1);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].angleFinal_1=atof(strp);
    printf("TngpV[N].angleFinal_1=%lf\n",TngpV[N].angleFinal_1);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].Nangles_1=atoi(strp);
    printf("TngpV[N].Nangles_1=%d\n",TngpV[N].Nangles_1);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].min_horizontal_scaling=atof(strp);
    printf("TngpV[N].min_horizontal_scaling=%lf\n",TngpV[N].min_horizontal_scaling);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].max_horizontal_scaling=atof(strp);
    printf("TngpV[N].max_horizontal_scaling=%lf\n",TngpV[N].max_horizontal_scaling);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].angleInit_2=atof(strp);
    printf("TngpV[N].angleInit_2=%lf\n",TngpV[N].angleInit_2);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].angleFinal_2=atof(strp);
    printf("TngpV[N].angleFinal_2=%lf\n",TngpV[N].angleFinal_2);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].Nangles_2=atoi(strp);
    printf("TngpV[N].Nangles_2=%d\n",TngpV[N].Nangles_2);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].min_uniform_scaling=atof(strp);
    printf("TngpV[N].min_uniform_scaling=%lf\n",TngpV[N].min_uniform_scaling);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].max_uniform_scaling=atof(strp);
    printf("TngpV[N].max_uniform_scaling=%lf\n",TngpV[N].max_uniform_scaling);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].inclusion_father=atoi(strp);
    printf("TngpV[N].inclusion_father=%d\n",TngpV[N].inclusion_father);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].exclusion_brothers=atoi(strp);
    printf("TngpV[N].exclusion_brothers=%d\n",TngpV[N].exclusion_brothers);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].exclusion_others=atoi(strp);
    printf("TngpV[N].exclusion_others=%d\n",TngpV[N].exclusion_others);

   ///RENDER PARAMETERS

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].random_seed_color=atoi(strp);
    printf("TngpV[N].random_seed_color=%d\n",TngpV[N].random_seed_color);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].border_thick=atof(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].palette_index=atoi(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].min_transparency=atof(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].max_transparency=atof(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].min_light_spot=atof(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].max_light_spot=atof(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].tesselation_palette_index=atoi(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].keepFatherInclusion=atoi(strp);
    printf("TngpV[N].keepFatherInclusion=%d\n",TngpV[N].keepFatherInclusion);

  }

   if(TngpV.size()==0){
     printf("TngpV.size()==0 in read_image_configuration_parameters()\n");
     { int ii_; scanf("%d",&ii_);} //system("pause");
   }

}

/// READ FROM DISK IMAGE PARAMETER CONFIGURATION
string read_image_configuration_parameters(
char name[],
shape &S,
float &width_float,
float &height_float,
vector<float> &border_thick,
vector<color> &background_color,
float &zoom,
vector<tree_node_generator_parameters> &TngpV,
int &OrderToDrawShapes,
float &scale,
string &DefaultPalette,
vector<int> &Database_Index,
vector<int> &Shape_Index)
{
  FILE *f;
  f = fopen (name , "r");
  if(f==NULL){
    printf("We can not open file : %s",name);
    //system("pause");
    { int ii_; scanf("%d",&ii_);}
  }

  TngpV.clear();

  char nod[200],strp[200],c='o';

  while(c!='=' && c!=EOF) {c=getc(f);  } c='o';
  fscanf(f,"%s",strp); //printf("strp = %s\n",strp); system("pause");
  if(isn(strp)==false) return string("image aspect ratio");
  width_float=atof(strp);
  if(width_float<0.1 || width_float>10.) return string("image aspect ratio has to be in the range [0.1,10.]");

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  //if(isn(strp)==false) return string("height_float");
  //height_float=atof(strp);
  height_float=1.;

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
    //printf("%s\n",strp);
    //printf("length stpr=%d\n",strlen(strp));
  border_thick.clear();
  for(int k=0;k<strlen(strp);k++){
     int m=0;
     while(strp[k]!=':' && k<strlen(strp)){
      nod[m++]=strp[k++];
     }
     nod[m]='\0';
     //printf("%s\n",nod);
     if(isn(nod)==true) border_thick.push_back(atof(nod));
     else border_thick.push_back(0.);
  }

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  DefaultPalette= string(strp);

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  //printf("%s\n",strp);
  //printf("length stpr=%d\n",strlen(strp));
  background_color.clear();
  for(int k=0;k<strlen(strp);k++){
     int m=0;
     while(strp[k]!=':' && k<strlen(strp)){
      nod[m++]=strp[k++];
     }
     nod[m]='\0';
     //printf("%s\n",nod);
     if(isn(nod)==true){
       background_color.resize(background_color.size()+1);
       background_color[background_color.size()-1].B_=(unsigned char) atoi(nod);
     }
  }
  int ncolors=0;
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  for(int k=0;k<strlen(strp);k++){
     int m=0;
     while(strp[k]!=':' && k<strlen(strp)){
      nod[m++]=strp[k++];
     }
     nod[m]='\0';
     if(isn(nod)==true){
       if(ncolors<background_color.size()){
         background_color[ncolors++].G_=(unsigned char) atoi(nod);
       }
     }
  }
  if(ncolors<background_color.size()){
    background_color.resize(ncolors);
  }
  ncolors=0;
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  for(int k=0;k<strlen(strp);k++){
     int m=0;
     while(strp[k]!=':' && k<strlen(strp)){
      nod[m++]=strp[k++];
     }
     nod[m]='\0';
     if(isn(nod)==true){
       if(ncolors<background_color.size()){
         background_color[ncolors++].R_=(unsigned char) atoi(nod);
       }
     }
  }
  if(ncolors<background_color.size()){
    background_color.resize(ncolors);
  }
//  for(int k=0;k<background_color.size();k++){
//    printf("c[%d]=(%d,%d,%d)\n",k,background_color[k].R_,background_color[k].G_,background_color[k].B_);
//  }
//  system("pause");

  /// The initial shape is a rectangle with borders
  //S=shape(triangulation(point2d(0.,0.),point2d(width_float,height_float)));
  //S.nodes.push_back(0);
  S.T=triangulation(); /** triangles defining the shape */
  S.extrema.clear(); /** all shape tree bounding box :  extrema[0]=(xmin,ymin) extrema[1]=(xmax,ymax) */
  S.extrema.push_back(point2d(0.,0.));
  S.extrema.push_back(point2d(width_float,height_float));
  S.shapeChild.clear(); /** children shapes */
  S.shapeBorder.clear(); /** Shapes which define the border */
  S.nodes.clear();
  float Sum_border_thick=0;
  for(int k=0;k<border_thick.size();k++){
    Sum_border_thick+=border_thick[k];
    if(Sum_border_thick>width_float*0.5 ||Sum_border_thick>height_float*0.5 ){
      border_thick.resize(k);
      break;
    }
  }
  point2d pmin=point2d(Sum_border_thick,Sum_border_thick);
  point2d pmax=point2d(width_float,height_float)+point2d(-Sum_border_thick,-Sum_border_thick);
  S.shapeChild.push_back(shape(triangulation(pmin,pmax)));
  S.shapeChild[0].nodes.push_back(0);
  S.shapeChild[0].keepFatherInclusion = 0;
  pmin=point2d(0.,0.);
  pmax=point2d(width_float,height_float);
  for(int k=0;k<border_thick.size();k++){
    if(border_thick[k]<=0.) continue;
    S.shapeChild.push_back(shape(triangulation(border_thick[k],pmin,pmax)));
    S.shapeChild[S.shapeChild.size()-1].nodes.push_back(k+1);
    pmin=pmin+point2d(border_thick[k],border_thick[k]);
    pmax=pmax-point2d(border_thick[k],border_thick[k]);
    //S.shapeChild[S.shapeChild.size()-1].c.random();
    S.shapeChild[S.shapeChild.size()-1].keepFatherInclusion = 0;
  }
  //S.print(); system("pause");


//  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
//  fscanf(f,"%s",strp);
//  unsigned char cB=atoi(strp);
//
//  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
//  fscanf(f,"%s",strp);
//  unsigned char cG=atoi(strp);
//
//  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
//  fscanf(f,"%s",strp);
//  unsigned char cR=atoi(strp);

  for(int k=0;k<background_color.size() && k<S.shapeChild.size();k++){ S.shapeChild[k].c=background_color[k]; }

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  if(isn(strp)==false) return string("light_spot");
  S.light_spot=atof(strp);
  for(int k=0;k<S.shapeChild.size();k++) S.shapeChild[k].light_spot=atof(strp);

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  if(isn(strp)==false) return string("light_spot center x");
  float px=atof(strp);

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  if(isn(strp)==false) return string("light_spot center y");
  float py=atof(strp);
  S.light_spot_center=point2d(px,py);
  for(int k=0;k<S.shapeChild.size();k++) S.shapeChild[k].light_spot_center=point2d(px,py);

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  if(isn(strp)==false) return string("image height");
  zoom=atof(strp);

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  if(isn(strp)==false) return string("Order to Draw shapes");
  OrderToDrawShapes=atoi(strp);

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  if(isn(strp)==false) return string("resolution");
  scale=atof(strp);
  if(scale<0.001) return string("resolution");
  printf("scale=%lf\n",scale);


  /// tree_node_generator_parameters
  while(true){
    while(c!='=' && c!=EOF) {c=getc(f);}
    if(c==EOF){
      fclose(f);
      return string("0");;
    }
    c='o';
    TngpV.resize(TngpV.size()+1);
    int N=TngpV.size()-1;

    fscanf(f,"%s",strp);
    //printf("%s\n",strp);
    //printf("length stpr=%d\n",strlen(strp));
    TngpV[N].nodes.clear();
    for(int k=0;k<strlen(strp);k++){
       int m=0;
       while(strp[k]!=':' && k<strlen(strp)){
        nod[m++]=strp[k++];
       }
       nod[m]='\0';
       //printf("%s\n",nod);
       if(isn(nod)==false) return string("node depth configuration");
       TngpV[N].nodes.push_back(atoi(nod));
    }
    for(int i=0;i<TngpV[N].nodes.size();i++) printf("%d ",TngpV[N].nodes[i]); //system("pause");

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("random seed shapes");
    TngpV[N].random_seed=atoi(strp);
    printf("TngpV[N].random_seed=%d\n",TngpV[N].random_seed);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if( isdigit(strp[0])==true || strp[0]=='-'){
      if(isn(strp)==false) return string("shape database");
      TngpV[N].shape_database=atoi(strp);
      printf("TngpV[N].shape_database=%d\n",TngpV[N].shape_database);

      while(c!='=' && c!=EOF) {c=getc(f);} c='o';
      fscanf(f,"%s",strp);
      if(isn(strp)==false) return string("shape index");
      TngpV[N].shape_index=atoi(strp);
      printf("TngpV[N].shape_index=%d\n",TngpV[N].shape_index);
    }
    else{
      while(c!='=' && c!=EOF) {c=getc(f);} c='o';
      fscanf(f,"%s",strp);
      if(isn(strp)==false) return string("shape index");
      TngpV[N].shape_database_and_index_definition(atoi(strp)%Shape_Index.size(),Database_Index,Shape_Index);
      printf("TngpV[N].shape_database=%d\n",TngpV[N].shape_database);
      printf("TngpV[N].shape_index=%d\n",TngpV[N].shape_index);
    }

//    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
//    fscanf(f,"%s",strp);
//    TngpV[N].max_number_shapes=atoi(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("shape diameter");
    if(atof(strp)<0.001) return string("shape diameter must be bigger than 0.001");
    TngpV[N].shape_diameter=atof(strp);
    printf("TngpV[N].shape_diameter=%lf\n",TngpV[N].shape_diameter);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("shape distribution type");
    TngpV[N].shape_distribution_type=atoi(strp);
    printf("TngpV[N].shape_distribution_type=%d\n",TngpV[N].shape_distribution_type);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("min distance contouring");
    TngpV[N].min_distance_contouring=atof(strp);
    printf("TngpV[N].min_distance_contouring=%lf\n",TngpV[N].min_distance_contouring);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("max distance contouring");
    TngpV[N].max_distance_contouring=atof(strp);
    printf("TngpV[N].max_distance_contouring=%lf\n",TngpV[N].max_distance_contouring);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("angle Init 1");
    TngpV[N].angleInit_1=atof(strp);
    printf("TngpV[N].angleInit_1=%lf\n",TngpV[N].angleInit_1);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("angle Final 1");
    TngpV[N].angleFinal_1=atof(strp);
    printf("TngpV[N].angleFinal_1=%lf\n",TngpV[N].angleFinal_1);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("Number of angles 1");
    TngpV[N].Nangles_1=atoi(strp);
    printf("TngpV[N].Nangles_1=%d\n",TngpV[N].Nangles_1);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("min horizontal scaling");
    TngpV[N].min_horizontal_scaling=atof(strp);
    if(TngpV[N].min_horizontal_scaling<0.001) return string("min horizontal scaling must be higher than 0.001");
    printf("TngpV[N].min_horizontal_scaling=%lf\n",TngpV[N].min_horizontal_scaling);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("max horizontal scaling");
    TngpV[N].max_horizontal_scaling=atof(strp);
    if(TngpV[N].max_horizontal_scaling<0.001) return string("max horizontal scaling must be higher than 0.001");
    printf("TngpV[N].max_horizontal_scaling=%lf\n",TngpV[N].max_horizontal_scaling);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("angle Init 2");
    TngpV[N].angleInit_2=atof(strp);
    printf("TngpV[N].angleInit_2=%lf\n",TngpV[N].angleInit_2);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("angle Final 2");
    TngpV[N].angleFinal_2=atof(strp);
    printf("TngpV[N].angleFinal_2=%lf\n",TngpV[N].angleFinal_2);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("Number of angles 2");
    TngpV[N].Nangles_2=atoi(strp);
    printf("TngpV[N].Nangles_2=%d\n",TngpV[N].Nangles_2);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("min uniform scaling");
    TngpV[N].min_uniform_scaling=atof(strp);
    if(TngpV[N].min_uniform_scaling<0.001) return string("min uniform scaling must be higher than 0.001");
    printf("TngpV[N].min_uniform_scaling=%lf\n",TngpV[N].min_uniform_scaling);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("max uniform scaling");
    TngpV[N].max_uniform_scaling=atof(strp);
    if(TngpV[N].max_uniform_scaling<0.001) return string("max uniform scaling must be higher than 0.001");
    printf("TngpV[N].max_uniform_scaling=%lf\n",TngpV[N].max_uniform_scaling);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("inclusion father");
    TngpV[N].inclusion_father=atoi(strp);
    printf("TngpV[N].inclusion_father=%d\n",TngpV[N].inclusion_father);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("exclusion brother");
    TngpV[N].exclusion_brothers=atoi(strp);
    printf("TngpV[N].exclusion_brothers=%d\n",TngpV[N].exclusion_brothers);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("exclusion others");
    TngpV[N].exclusion_others=atoi(strp);
    printf("TngpV[N].exclusion_others=%d\n",TngpV[N].exclusion_others);

   ///RENDER PARAMETERS

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("random seed color");
    TngpV[N].random_seed_color=atoi(strp);
    printf("TngpV[N].random_seed_color=%d\n",TngpV[N].random_seed_color);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("border thick");
    TngpV[N].border_thick=atof(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("palette index");
    TngpV[N].palette_index=atoi(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("min transparency");
    TngpV[N].min_transparency=atof(strp);
    if(TngpV[N].min_transparency<0 || TngpV[N].min_transparency>1) return string("min transparency must be in the range [0,1]");

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("max transparency");
    TngpV[N].max_transparency=atof(strp);
    if(TngpV[N].max_transparency<0 || TngpV[N].max_transparency>1) return string("max transparency must be in the range [0,1]");

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("min light spot");
    TngpV[N].min_light_spot=atof(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("max light spot");
    TngpV[N].max_light_spot=atof(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("tesselation palette index");
    TngpV[N].tesselation_palette_index=atoi(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    if(isn(strp)==false) return string("Keep father inclusion");
    TngpV[N].keepFatherInclusion=atoi(strp);
    printf("TngpV[N].keepFatherInclusion=%d\n",TngpV[N].keepFatherInclusion);

  }

   if(TngpV.size()==0){
     printf("TngpV.size()==0 in read_image_configuration_parameters()\n");
     return string("There is not any node in the tree");
   }

   return string("0");
}

/// READ FROM DISK IMAGE PARAMETER CONFIGURATION
int read_image_configuration_parametersV0(
char name[],
shape &S,
float &width_float,
float &height_float,
vector<float> &border_thick,
vector<color> &background_color,
float &zoom,
vector<tree_node_generator_parameters> &TngpV,
int &OrderToDrawShapes,
float &scale,
string &DefaultPalette)
{
  FILE *f;
  f = fopen (name , "r");
  if(f==NULL){
    printf("We can not open file : %s",name);
    //system("pause");
    { int ii_; scanf("%d",&ii_);}
  }

  TngpV.clear();

  char nod[200],strp[200],c='o';

  while(c!='=' && c!=EOF) {c=getc(f);  } c='o';
  fscanf(f,"%s",strp); //printf("strp = %s\n",strp); system("pause");
  width_float=atof(strp);

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  height_float=atof(strp);

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
    //printf("%s\n",strp);
    //printf("length stpr=%d\n",strlen(strp));
  border_thick.clear();
  for(int k=0;k<strlen(strp);k++){
     int m=0;
     while(strp[k]!=':' && k<strlen(strp)){
      nod[m++]=strp[k++];
     }
     nod[m]='\0';
     //printf("%s\n",nod);
     border_thick.push_back(atof(nod));
  }

//  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
//  fscanf(f,"%s",strp);
//  DefaultPalette= string(strp);

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  //printf("%s\n",strp);
  //printf("length stpr=%d\n",strlen(strp));
  background_color.clear();
  for(int k=0;k<strlen(strp);k++){
     int m=0;
     while(strp[k]!=':' && k<strlen(strp)){
      nod[m++]=strp[k++];
     }
     nod[m]='\0';
     //printf("%s\n",nod);
     background_color.resize(background_color.size()+1);
     background_color[background_color.size()-1].B_=(unsigned char) atoi(nod);
  }
  int ncolors=0;
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  for(int k=0;k<strlen(strp);k++){
     int m=0;
     while(strp[k]!=':' && k<strlen(strp)){
      nod[m++]=strp[k++];
     }
     nod[m]='\0';
     if(ncolors<background_color.size()){
       background_color[ncolors++].G_=(unsigned char) atoi(nod);
     }
  }
  ncolors=0;
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  for(int k=0;k<strlen(strp);k++){
     int m=0;
     while(strp[k]!=':' && k<strlen(strp)){
      nod[m++]=strp[k++];
     }
     nod[m]='\0';
     if(ncolors<background_color.size()){
       background_color[ncolors++].R_=(unsigned char) atoi(nod);
     }
  }
//  for(int k=0;k<background_color.size();k++){
//    printf("c[%d]=(%d,%d,%d)\n",k,background_color[k].R_,background_color[k].G_,background_color[k].B_);
//  }
//  system("pause");

  /// The initial shape is a rectangle with borders
  //S=shape(triangulation(point2d(0.,0.),point2d(width_float,height_float)));
  //S.nodes.push_back(0);
  S.T=triangulation(); /** triangles defining the shape */
  S.extrema.clear(); /** all shape tree bounding box :  extrema[0]=(xmin,ymin) extrema[1]=(xmax,ymax) */
  S.extrema.push_back(point2d(0.,0.));
  S.extrema.push_back(point2d(width_float,height_float));
  S.shapeChild.clear(); /** children shapes */
  S.shapeBorder.clear(); /** Shapes which define the border */
  S.nodes.clear();
  float Sum_border_thick=0;
  for(int k=0;k<border_thick.size();k++) Sum_border_thick+=border_thick[k];
  point2d pmin=point2d(Sum_border_thick,Sum_border_thick);
  point2d pmax=point2d(width_float,height_float)+point2d(-Sum_border_thick,-Sum_border_thick);
  S.shapeChild.push_back(shape(triangulation(pmin,pmax)));
  S.shapeChild[0].nodes.push_back(0);
  S.shapeChild[0].keepFatherInclusion = 0;
  pmin=point2d(0.,0.);
  pmax=point2d(width_float,height_float);
  for(int k=0;k<border_thick.size();k++){
    if(border_thick[k]<=0.) continue;
    S.shapeChild.push_back(shape(triangulation(border_thick[k],pmin,pmax)));
    S.shapeChild[S.shapeChild.size()-1].nodes.push_back(k+1);
    pmin=pmin+point2d(border_thick[k],border_thick[k]);
    pmax=pmax-point2d(border_thick[k],border_thick[k]);
    //S.shapeChild[S.shapeChild.size()-1].c.random();
    S.shapeChild[S.shapeChild.size()-1].keepFatherInclusion = 0;
  }
  //S.print(); system("pause");


//  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
//  fscanf(f,"%s",strp);
//  unsigned char cB=atoi(strp);
//
//  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
//  fscanf(f,"%s",strp);
//  unsigned char cG=atoi(strp);
//
//  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
//  fscanf(f,"%s",strp);
//  unsigned char cR=atoi(strp);

  for(int k=0;k<background_color.size() && k<S.shapeChild.size();k++){ S.shapeChild[k].c=background_color[k]; }

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  S.light_spot=atof(strp);
  for(int k=0;k<S.shapeChild.size();k++) S.shapeChild[k].light_spot=atof(strp);

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  float px=atof(strp);

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  float py=atof(strp);
  S.light_spot_center=point2d(px,py);
  for(int k=0;k<S.shapeChild.size();k++) S.shapeChild[k].light_spot_center=point2d(px,py);

  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);
  zoom=atof(strp);

//  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
//  fscanf(f,"%s",strp);
//  OrderToDrawShapes=atoi(strp);
//
//  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
//  fscanf(f,"%s",strp);
//  scale=atof(strp);
//  printf("scale=%lf\n",scale);


  /// tree_node_generator_parameters
  while(true){
    while(c!='=' && c!=EOF) {c=getc(f);}
    if(c==EOF){
      fclose(f);
      return -1;
    }
    c='o';
    TngpV.resize(TngpV.size()+1);
    int N=TngpV.size()-1;

    fscanf(f,"%s",strp);
    //printf("%s\n",strp);
    //printf("length stpr=%d\n",strlen(strp));
    TngpV[N].nodes.clear();
    for(int k=0;k<strlen(strp);k++){
       int m=0;
       while(strp[k]!=':' && k<strlen(strp)){
        nod[m++]=strp[k++];
       }
       nod[m]='\0';
       //printf("%s\n",nod);
       TngpV[N].nodes.push_back(atoi(nod));
    }
    for(int i=0;i<TngpV[N].nodes.size();i++) printf("%d ",TngpV[N].nodes[i]); //system("pause");

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].random_seed=atoi(strp);
    printf("TngpV[N].random_seed=%d\n",TngpV[N].random_seed);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].shape_database=atoi(strp);
    printf("TngpV[N].shape_database=%d\n",TngpV[N].shape_database);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].shape_index=atoi(strp);
    printf("TngpV[N].shape_index=%d\n",TngpV[N].shape_index);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
//    TngpV[N].max_number_shapes=atoi(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].shape_diameter=atof(strp);
    printf("TngpV[N].shape_diameter=%lf\n",TngpV[N].shape_diameter);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].shape_distribution_type=atoi(strp);
    printf("TngpV[N].shape_distribution_type=%d\n",TngpV[N].shape_distribution_type);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].angleInit_1=atof(strp);
    printf("TngpV[N].angleInit_1=%lf\n",TngpV[N].angleInit_1);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].angleFinal_1=atof(strp);
    printf("TngpV[N].angleFinal_1=%lf\n",TngpV[N].angleFinal_1);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].Nangles_1=atoi(strp);
    printf("TngpV[N].Nangles_1=%d\n",TngpV[N].Nangles_1);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].angleInit_2=atof(strp);
    printf("TngpV[N].angleInit_2=%lf\n",TngpV[N].angleInit_2);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].angleFinal_2=atof(strp);
    printf("TngpV[N].angleFinal_2=%lf\n",TngpV[N].angleFinal_2);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].Nangles_2=atoi(strp);
    printf("TngpV[N].Nangles_2=%d\n",TngpV[N].Nangles_2);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].inclusion_father=atoi(strp);
    printf("TngpV[N].inclusion_father=%d\n",TngpV[N].inclusion_father);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].exclusion_brothers=atoi(strp);
    printf("TngpV[N].exclusion_brothers=%d\n",TngpV[N].exclusion_brothers);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].exclusion_others=atoi(strp);
    printf("TngpV[N].exclusion_others=%d\n",TngpV[N].exclusion_others);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].min_uniform_scaling=atof(strp);
    printf("TngpV[N].min_uniform_scaling=%lf\n",TngpV[N].min_uniform_scaling);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].max_uniform_scaling=atof(strp);
    printf("TngpV[N].max_uniform_scaling=%lf\n",TngpV[N].max_uniform_scaling);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].min_horizontal_scaling=atof(strp);
    printf("TngpV[N].min_horizontal_scaling=%lf\n",TngpV[N].min_horizontal_scaling);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].max_horizontal_scaling=atof(strp);
    printf("TngpV[N].max_horizontal_scaling=%lf\n",TngpV[N].max_horizontal_scaling);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].min_distance_contouring=atof(strp);
    printf("TngpV[N].min_distance_contouring=%lf\n",TngpV[N].min_distance_contouring);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].max_distance_contouring=atof(strp);
    printf("TngpV[N].max_distance_contouring=%lf\n",TngpV[N].max_distance_contouring);



   ///RENDER PARAMETERS

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].border_thick=atof(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].random_seed_color=atoi(strp);
    printf("TngpV[N].random_seed_color=%d\n",TngpV[N].random_seed_color);



    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].palette_index=atoi(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].min_transparency=atof(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].max_transparency=atof(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].min_light_spot=atof(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].max_light_spot=atof(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].tesselation_palette_index=atoi(strp);

    while(c!='=' && c!=EOF) {c=getc(f);} c='o';
    fscanf(f,"%s",strp);
    TngpV[N].keepFatherInclusion=atoi(strp);
    printf("TngpV[N].keepFatherInclusion=%d\n",TngpV[N].keepFatherInclusion);

  }

   if(TngpV.size()==0){
     printf("TngpV.size()==0 in read_image_configuration_parameters()\n");
     { int ii_; scanf("%d",&ii_);} //system("pause");
   }

}


shape RandomShapeParameterConfiguration(
float &width_float,
float &height_float,
vector<float> &border_thick,
vector<color> &background_color,
vector<tree_node_generator_parameters> &TngpV,
 vector<imageBMP> &PaletteDataBase,
int palette_index,
int &OrderToDrawShapes,
float &scale,
bool periodicity)
{
  Ncall_tree_node_generator=0;
  border_thick.clear();
  background_color.clear();
  TngpV.clear();

  shape S;
  S.extrema.push_back(point2d(0.,0.));
  S.extrema.push_back(point2d(width_float,height_float));

  srand (time(NULL));
  rand();

  if(ami_rand<0.5 && periodicity==false){
    border_thick.push_back(0.015+ami_rand*0.02);
    border_thick.push_back(2.*border_thick[0]+ami_rand*(0.12-2.*border_thick[0]));
    border_thick.push_back(border_thick[0]);
    if(ami_rand<0.5){
      border_thick.push_back(border_thick[1]*0.7+0.3*ami_rand*border_thick[1]);
      border_thick.push_back(border_thick[0]);
    }
  }
  //printf("border_thick.size()=%d\n",border_thick.size());
  for(int k=0;k<border_thick.size()+1;k++) background_color.push_back(PaletteDataBase[palette_index%PaletteDataBase.size()][ami_randI(PaletteDataBase[palette_index%PaletteDataBase.size()].size())]);

  float Sum_border_thick=0;
  for(int k=0;k<border_thick.size();k++) Sum_border_thick+=border_thick[k];
  point2d pmin=point2d(Sum_border_thick,Sum_border_thick);
  point2d pmax=point2d(width_float,height_float)+point2d(-Sum_border_thick,-Sum_border_thick);
  S.shapeChild.push_back(shape(triangulation(pmin,pmax)));
  S.shapeChild[0].nodes.push_back(0);
  S.shapeChild[0].keepFatherInclusion = 0;
  pmin=point2d(0.,0.);
  pmax=point2d(width_float,height_float);
  for(int k=0;k<border_thick.size();k++){
    if(border_thick[k]<=0.) continue;
    S.shapeChild.push_back(shape(triangulation(border_thick[k],pmin,pmax)));
    S.shapeChild[S.shapeChild.size()-1].nodes.push_back(k+1);
    pmin=pmin+point2d(border_thick[k],border_thick[k]);
    pmax=pmax-point2d(border_thick[k],border_thick[k]);
    S.shapeChild[S.shapeChild.size()-1].c.random();
    S.shapeChild[S.shapeChild.size()-1].keepFatherInclusion = 0;
    S.shapeChild[S.shapeChild.size()-1].extrema_update();
  }

  for(int k=0;k<background_color.size() && k<S.shapeChild.size();k++){ S.shapeChild[k].c=background_color[k]; }

  S.light_spot=ami_rand<0.5?-1.:0.2+0.6*ami_rand;
  if(periodicity==true) S.light_spot=-1;
  for(int k=0;k<S.shapeChild.size();k++) S.shapeChild[k].light_spot=S.light_spot;

  point2d pl(ami_rand*width_float,ami_rand*height_float);
  S.light_spot_center=pl;
  for(int k=0;k<S.shapeChild.size();k++) S.shapeChild[k].light_spot_center=pl;
//printf("ok1\n");
  /// TREE NODES OF CENTRAL RECTANGLE
  int NtreeNodes0=1+rand()%3;
  float shape_diameter=0.1+0.2*ami_rand;
  if(NtreeNodes0==1) shape_diameter=0.05+0.1*ami_rand;
  else if(NtreeNodes0==2) shape_diameter=0.075+0.1*ami_rand;
  for(int k=0;k< NtreeNodes0;k++){
    tree_node_generator_parameters tngp;
    vector<short> nodes(1,0);
    nodes.push_back(k);
    tngp.nodes=nodes;
    tngp.shape_diameter=shape_diameter;
    if(ami_rand<0.2){
      tngp.shape_distribution_type=6;
      tngp.shape_diameter=shape_diameter/2;
    }
    if(tngp.palette_index!=-1) tngp.palette_index=palette_index;
    TngpV.push_back(tngp);
    shape_diameter=shape_diameter*(2.*ami_rand+1.)/3.;
  }
  if(ami_rand<0.5){
    tree_node_generator_parameters tngp;
    vector<short> nodes(3,0);
    tngp.nodes=nodes;
    tngp.shape_diameter=TngpV[0].shape_diameter*(0.2+0.3*ami_rand);
    if(tngp.palette_index!=-1) tngp.palette_index=palette_index;
    TngpV.push_back(tngp);
    if(ami_rand<0.5){
      tree_node_generator_parameters tngp;
      vector<short> nodes(2,0);
      nodes.push_back(1);
      tngp.nodes=nodes;
      tngp.shape_diameter=TngpV[0].shape_diameter*(0.2+0.3*ami_rand);//*(2.*ami_rand+1.)/3.;
      if(tngp.palette_index!=-1) tngp.palette_index=palette_index;
      TngpV.push_back(tngp);
    }
  }
  if(periodicity==true) TngpV[0].inclusion_father=-1;
  int sizeTngpV=TngpV.size();
//printf("ok2\n");
  /// TREE NODES OF BORDER STRIPES
  for(int k=0;k<border_thick.size();k++){
    tree_node_generator_parameters tngp;
    tngp.shape_distribution_type=4;
    tngp.keepFatherInclusion=0;
    tngp.inclusion_father=-1;
    tngp.exclusion_others=-1;
    tngp.exclusion_brothers=-1;
    tngp.border_thick=0.;
    if(k>1){
      tngp=TngpV[TngpV.size()-2];
    }
    else{
      if(tngp.palette_index!=-1) tngp.palette_index=palette_index;
    }

    tngp.shape_diameter=border_thick[k];
    vector<short> nodes(1,k+1);
    nodes.push_back(0);
    tngp.nodes=nodes;
    TngpV.push_back(tngp);
  }
  if(TngpV.size()>0 && TngpV[0].palette_index>=0 ){
    for(int k=1;k<TngpV.size();k++){
      TngpV[k].palette_index=TngpV[0].palette_index;
      if(TngpV[k].tesselation_palette_index>=0) TngpV[k].tesselation_palette_index=TngpV[0].palette_index;
    }
  }
//printf("ok3\n");

  OrderToDrawShapes=ami_rand<0.5?0:-1;
  scale=(0.5+4.*ami_rand*ami_rand)*sqrt((double)sizeTngpV);
  if(periodicity==true) scale*=0.5;

  return S;
}

void AddBorder(
shape &S,
float &width_float,
float &height_float,
vector<float> &border_thick,
vector<color> &background_color,
vector<tree_node_generator_parameters> &TngpV,
vector<imageBMP> &PaletteDataBase,
int palette_index
)
{
  srand (time(NULL)); rand();
  //printf("background_color.size()=%d\n",background_color.size()); system("pause");
  //border_thick[0]=(0.015+ami_rand*0.02);
  border_thick[0]=0.015;
  //border_thick.push_back(2.*border_thick[0]+ami_rand*(0.12-2.*border_thick[0]));
  border_thick.push_back(0.030);
  border_thick.push_back(border_thick[0]);
  //printf("border_thick.size()=%d\n",border_thick.size());
  for(int k=0;k<border_thick.size();k++) background_color.push_back(PaletteDataBase[palette_index%PaletteDataBase.size()][ami_randI(PaletteDataBase[palette_index%PaletteDataBase.size()].size())]);

  float Sum_border_thick=0;
  for(int k=0;k<border_thick.size();k++) Sum_border_thick+=border_thick[k];
  point2d pmin=point2d(Sum_border_thick,Sum_border_thick);
  point2d pmax=point2d(width_float,height_float)+point2d(-Sum_border_thick,-Sum_border_thick);
  S.shapeChild.push_back(shape(triangulation(pmin,pmax)));
  S.shapeChild[0].T=triangulation(pmin,pmax);

  pmin=point2d(0.,0.);
  pmax=point2d(width_float,height_float);
  for(int k=0;k<border_thick.size();k++){
    if(border_thick[k]<=0.) continue;
    S.shapeChild.push_back(shape(triangulation(border_thick[k],pmin,pmax)));
    S.shapeChild[S.shapeChild.size()-1].nodes.push_back(k+1);
    pmin=pmin+point2d(border_thick[k],border_thick[k]);
    pmax=pmax-point2d(border_thick[k],border_thick[k]);
    S.shapeChild[S.shapeChild.size()-1].c.random();
    S.shapeChild[S.shapeChild.size()-1].keepFatherInclusion = 0;
    S.shapeChild[S.shapeChild.size()-1].extrema_update();
  }

  for(int k=1;k<background_color.size() && k<S.shapeChild.size();k++){ S.shapeChild[k].c=background_color[k]; }

  for(int k=1;k<S.shapeChild.size();k++) S.shapeChild[k].light_spot=S.shapeChild[0].light_spot;

  for(int k=1;k<S.shapeChild.size();k++) S.shapeChild[k].light_spot_center=S.shapeChild[0].light_spot_center;

//printf("ok2\n");
  /// TREE NODES OF BORDER STRIPES
  for(int k=0;k<border_thick.size();k++){
    tree_node_generator_parameters tngp;
    tngp.shape_distribution_type=4;
    tngp.keepFatherInclusion=0;
    tngp.inclusion_father=-1;
    tngp.exclusion_others=-1;
    tngp.exclusion_brothers=-1;
    tngp.border_thick=0.;
    if(k>1){
      tngp=TngpV[TngpV.size()-2];
    }
    else{
      if(tngp.palette_index!=-1) tngp.palette_index=palette_index;
    }

    tngp.shape_diameter=border_thick[k];
    vector<short> nodes(1,k+1);
    nodes.push_back(0);
    tngp.nodes=nodes;
    TngpV.push_back(tngp);
  }
  if(TngpV.size()>0 && TngpV[0].palette_index>=0 ){
    for(int k=1;k<TngpV.size();k++){
      TngpV[k].palette_index=TngpV[0].palette_index;
      if(TngpV[k].tesselation_palette_index>=0) TngpV[k].tesselation_palette_index=TngpV[0].palette_index;
    }
  }

}

void tree_generator(shape &S,vector < vector<shape> > &ShapeDataBaseV,vector<imageBMP> &PaletteDataBase,
                      vector<tree_node_generator_parameters> &TngpV,float scale)
{
  Ncall_tree_node_generator=0;
  Ncall_tree_node_generator_parameters=0;
  for(int k=0;k<TngpV.size();k++){
     if(scale>1e-20){
        TngpV[k].shape_diameter*=scale;
        TngpV[k].border_thick*=scale;
        TngpV[k].min_distance_contouring*=scale;
        TngpV[k].max_distance_contouring*=scale;
     }
     printf("k=%d Nnodes=%d  node  ",k,TngpV.size());
     for(int m=0;m<TngpV[k].nodes.size()-1;m++) printf("%d:",TngpV[k].nodes[m]);
     printf("%d  shape_diameter=%lf\n",TngpV[k].nodes[TngpV[k].nodes.size()-1],TngpV[k].shape_diameter);
     TngpV[k].tree_node_generator(S,ShapeDataBaseV,PaletteDataBase);
  }
}

